/****************************************************************************
** Meta object code from reading C++ file 'interface.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.9.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/libkipi-17.08.3/src/interface.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.9.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KIPI__Interface_t {
    QByteArrayData data[13];
    char stringdata0[172];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KIPI__Interface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KIPI__Interface_t qt_meta_stringdata_KIPI__Interface = {
    {
QT_MOC_LITERAL(0, 0, 15), // "KIPI::Interface"
QT_MOC_LITERAL(1, 16, 16), // "selectionChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 12), // "hasSelection"
QT_MOC_LITERAL(4, 47, 19), // "currentAlbumChanged"
QT_MOC_LITERAL(5, 67, 12), // "gotThumbnail"
QT_MOC_LITERAL(6, 80, 10), // "gotPreview"
QT_MOC_LITERAL(7, 91, 16), // "progressCanceled"
QT_MOC_LITERAL(8, 108, 2), // "id"
QT_MOC_LITERAL(9, 111, 17), // "reservedForAction"
QT_MOC_LITERAL(10, 129, 3), // "url"
QT_MOC_LITERAL(11, 133, 19), // "descriptionOfAction"
QT_MOC_LITERAL(12, 153, 18) // "reservationCleared"

    },
    "KIPI::Interface\0selectionChanged\0\0"
    "hasSelection\0currentAlbumChanged\0"
    "gotThumbnail\0gotPreview\0progressCanceled\0"
    "id\0reservedForAction\0url\0descriptionOfAction\0"
    "reservationCleared"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KIPI__Interface[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       4,    1,   52,    2, 0x06 /* Public */,
       5,    2,   55,    2, 0x06 /* Public */,
       6,    2,   60,    2, 0x06 /* Public */,
       7,    1,   65,    2, 0x06 /* Public */,
       9,    2,   68,    2, 0x06 /* Public */,
      12,    1,   73,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QPixmap,    2,    2,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QImage,    2,    2,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QUrl, QMetaType::QString,   10,   11,
    QMetaType::Void, QMetaType::QUrl,   10,

       0        // eod
};

void KIPI::Interface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Interface *_t = static_cast<Interface *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->selectionChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->currentAlbumChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->gotThumbnail((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QPixmap(*)>(_a[2]))); break;
        case 3: _t->gotPreview((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QImage(*)>(_a[2]))); break;
        case 4: _t->progressCanceled((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->reservedForAction((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->reservationCleared((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (Interface::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Interface::selectionChanged)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (Interface::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Interface::currentAlbumChanged)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (Interface::*_t)(const QUrl & , const QPixmap & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Interface::gotThumbnail)) {
                *result = 2;
                return;
            }
        }
        {
            typedef void (Interface::*_t)(const QUrl & , const QImage & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Interface::gotPreview)) {
                *result = 3;
                return;
            }
        }
        {
            typedef void (Interface::*_t)(const QString & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Interface::progressCanceled)) {
                *result = 4;
                return;
            }
        }
        {
            typedef void (Interface::*_t)(const QUrl & , const QString & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Interface::reservedForAction)) {
                *result = 5;
                return;
            }
        }
        {
            typedef void (Interface::*_t)(const QUrl & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Interface::reservationCleared)) {
                *result = 6;
                return;
            }
        }
    }
}

const QMetaObject KIPI::Interface::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_KIPI__Interface.data,
      qt_meta_data_KIPI__Interface,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *KIPI::Interface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KIPI::Interface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KIPI__Interface.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KIPI::Interface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KIPI::Interface::selectionChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KIPI::Interface::currentAlbumChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KIPI::Interface::gotThumbnail(const QUrl & _t1, const QPixmap & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KIPI::Interface::gotPreview(const QUrl & _t1, const QImage & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void KIPI::Interface::progressCanceled(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void KIPI::Interface::reservedForAction(const QUrl & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void KIPI::Interface::reservationCleared(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
